#include "zeemainwindow.h"

#include "zeeexportdialog.h"
#include "zeeimportdialog.h"
#include "zeenewdialog.h"
#include "zeeopendialog.h"
#include "zeeprogressdialog.h"
#include "zeerecipeeditwindow.h"
#include "zeerecipeviewwindow.h"
#include "zeerecipelistview.h"
#include "zeesearchdialog.h"
#include "zeestartupdialog.h"
#include "zeeutil.h"

#include "sql/qsqldatabase.h"
#include "sql/qsqlquery.h"

#include <qpe/applnk.h>
#include <qpe/config.h>
#include <qpe/filemanager.h>
#include <qpe/ir.h>
#include <qpe/qpemenubar.h>
#include <qpe/resource.h>

#include <qaction.h>
#include <qapplication.h>
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qtextstream.h>
#include <qtimer.h>

#include <stdlib.h>

ZeeMainWindow::ZeeMainWindow(QWidget* parent, const char* name)
    : QWidget(parent, name)
{
    m_pDatabase = QSqlDatabase::addDatabase("QSQLITE2");

    setCaption(tr("Zee Cookbook"));

    QPEMenuBar* menuBar = new QPEMenuBar(this);

    QPopupMenu* CookbookPopupMenu = new QPopupMenu(menuBar);

    m_pNewCookbookAction = new QAction(tr("New"), Resource::loadIconSet("zeecookbook/new"), tr("New..."), 0, this);
    connect(m_pNewCookbookAction, SIGNAL(activated()), this, SLOT(newCookbook()));
    m_pNewCookbookAction->addTo(CookbookPopupMenu);

    m_pOpenCookbookAction = new QAction(tr("Open"), Resource::loadIconSet("zeecookbook/open"), tr("Open..."), 0, this);
    connect(m_pOpenCookbookAction, SIGNAL(activated()), this, SLOT(openCookbook()));
    m_pOpenCookbookAction->addTo(CookbookPopupMenu);

    menuBar->insertItem(tr("Cookbook"), CookbookPopupMenu);

    QPopupMenu* recipePopupMenu = new QPopupMenu(menuBar);

    m_pAddRecipeAction = new QAction(tr("Add"), Resource::loadIconSet("zeecookbook/add"), tr("Add..."), 0, this);
    connect(m_pAddRecipeAction, SIGNAL(activated()), this, SLOT(addRecipe()));
    m_pAddRecipeAction->addTo(recipePopupMenu);

    m_pEditRecipeAction = new QAction(tr("Edit"), Resource::loadIconSet("zeecookbook/edit"), tr("Edit..."), 0, this);
    connect(m_pEditRecipeAction, SIGNAL(activated()), this, SLOT(editRecipe()));
    m_pEditRecipeAction->addTo(recipePopupMenu);

    m_pRemoveRecipeAction = new QAction(tr("Remove"), Resource::loadIconSet("zeecookbook/remove"), QString::null, 0, this);
    connect(m_pRemoveRecipeAction, SIGNAL(activated()), this, SLOT(removeRecipe()));
    m_pRemoveRecipeAction->addTo(recipePopupMenu);
    
    recipePopupMenu->insertSeparator();

    m_pViewRecipeAction = new QAction(tr("View"), Resource::loadIconSet("zeecookbook/view"), tr("View..."), 0, this);
    connect(m_pViewRecipeAction, SIGNAL(activated()), this, SLOT(viewRecipe()));
    m_pViewRecipeAction->addTo(recipePopupMenu);

    m_pBeamRecipeAction = new QAction(tr("Beam"), Resource::loadIconSet("zeecookbook/beam"), tr("Beam..."), 0, this);
    connect(m_pBeamRecipeAction, SIGNAL(activated()), this, SLOT(beamRecipe()));
    m_pBeamRecipeAction->addTo(recipePopupMenu);

    menuBar->insertItem(tr("Recipe"), recipePopupMenu);

    QPopupMenu* toolPopupMenu = new QPopupMenu(menuBar);

    m_pSearchToolAction = new QAction(tr("Search"), Resource::loadIconSet("zeecookbook/search"), tr("Search..."), 0, this);
    connect(m_pSearchToolAction, SIGNAL(activated()), this, SLOT(searchTool()));
    m_pSearchToolAction->addTo(toolPopupMenu);

    m_pShowAllToolAction = new QAction(tr("Show All"), QString::null, 0, this);
    connect(m_pShowAllToolAction, SIGNAL(activated()), this, SLOT(showAllTool()));
    m_pShowAllToolAction->addTo(toolPopupMenu);

    toolPopupMenu->insertSeparator();

    m_pImportToolAction = new QAction(tr("Import"), Resource::loadIconSet("zeecookbook/import"), tr("Import..."), 0, this);
    connect(m_pImportToolAction, SIGNAL(activated()), this, SLOT(importTool()));
    m_pImportToolAction->addTo(toolPopupMenu);

    m_pExportToolAction = new QAction(tr("Export"), Resource::loadIconSet("zeecookbook/export"), tr("Export..."), 0, this);
    connect(m_pExportToolAction, SIGNAL(activated()), this, SLOT(exportTool()));
    m_pExportToolAction->addTo(toolPopupMenu);

    menuBar->insertItem(tr("Tools"), toolPopupMenu);

    m_pRecipeListView = new ZeeRecipeListView(this);
    connect(m_pRecipeListView, SIGNAL(selectionChanged()), this, SLOT(selectRecipe()));

    m_pStatusLabel = new QLabel(this);

    QVBoxLayout* mainLayout = new QVBoxLayout(this, 4, 4);

    mainLayout->setMenuBar(menuBar);
    mainLayout->addWidget(m_pRecipeListView);
    mainLayout->addWidget(m_pStatusLabel);

    updateView();

    QTimer::singleShot(0, this, SLOT(startup()));
}

ZeeMainWindow::~ZeeMainWindow()
{
}

void ZeeMainWindow::updateView()
{
    blockSignals(true);

    bool hasDatabase = (m_pDatabase->isOpen());
    bool isRecipeSelected = (m_pRecipeListView->selected() != -1);
    bool hasRecipe = false;

    if (hasDatabase) {
        QString queryString = "SELECT COUNT(*) FROM recipe";
        QSqlQuery query(queryString);

        if (query.first()) {
            int count = query.value(0).toInt();
            hasRecipe = (count > 0);
        }
    }

    bool hasFilter = (!m_pRecipeListView->filter().isNull());
    int displayCount = m_pRecipeListView->childCount();

    m_pAddRecipeAction->setEnabled(hasDatabase);
    m_pEditRecipeAction->setEnabled(hasDatabase && isRecipeSelected);
    m_pRemoveRecipeAction->setEnabled(hasDatabase && isRecipeSelected);
    m_pViewRecipeAction->setEnabled(hasDatabase && isRecipeSelected);
    m_pBeamRecipeAction->setEnabled(hasDatabase && isRecipeSelected);
    m_pSearchToolAction->setEnabled(hasDatabase && hasRecipe);
    m_pShowAllToolAction->setEnabled(hasDatabase && hasFilter);
    m_pImportToolAction->setEnabled(hasDatabase);
    m_pExportToolAction->setEnabled(hasDatabase && hasRecipe);

    if (hasDatabase) {
        m_pStatusLabel->setText(QString::number(displayCount) + " " + tr("recipe(s)"));
    } else {
        m_pStatusLabel->setText(tr("No cookbook opened"));
    }

    blockSignals(false);
}

void ZeeMainWindow::setDocument(const QString& fileName)
{
    DocLnk docLnk(fileName, true);

    if (m_pDatabase->isOpen()) {
        m_pDatabase->close();
    }

    m_pDatabase->setDatabaseName(docLnk.file());

    if (m_pDatabase->open()) {
        Config config("zeecookbook");
        config.setGroup("General");
        config.writeEntry("LastCookbook", docLnk.file());
    }

    m_pRecipeListView->updateView();
    updateView();
}

void ZeeMainWindow::startup()
{
    ZeeStartupDialog startupDialog(this);

    if (startupDialog.exec() == QDialog::Accepted) {
        switch (startupDialog.selected()) {
            case ZeeStartupDialog::Open:
                openCookbook();
                break;
            case ZeeStartupDialog::Last:
                openLastCookbook();
                break;
            default:
                newCookbook();
                break;
        }
    }
}

void ZeeMainWindow::newCookbook()
{
    ZeeNewDialog newDialog(this);

    if (newDialog.exec() == QDialog::Accepted) {
        QString name = newDialog.selectedName();
        QString location = newDialog.selectedLocation();

        DocLnk newDocLnk;
        newDocLnk.setType("application/zeecookbook");
        newDocLnk.setName(name);

        QString fileName = location +  "/Documents/" + newDocLnk.type() + "/" + name.replace(QRegExp("/"), "_");

        if (QFile::exists(fileName) || QFile::exists(fileName + ".desktop")) {
            int i = 1;
            QString newFileName = fileName + "_" + QString::number(i);

            while (QFile::exists(newFileName) || QFile::exists(newFileName + ".desktop")) {
                i++;
                newFileName = fileName + "_" + QString::number(i);
            }

            fileName = newFileName;
        }

        newDocLnk.setFile(fileName);
        newDocLnk.setLinkFile(fileName + ".desktop");
        newDocLnk.writeLink();
        
        if (m_pDatabase->isOpen()) {
           m_pDatabase->close();
        }

        m_pDatabase->setDatabaseName(newDocLnk.file());

        if (m_pDatabase->open()) {
            QString queryString = "CREATE TABLE recipe (id INTEGER PRIMARY KEY, name, yield_quantity, yield_id, direction)";
            QSqlQuery query(queryString);

            queryString = "CREATE TABLE yield (id INTEGER PRIMARY KEY, name)";
            query.exec(queryString);

            queryString = "CREATE TABLE category (id INTEGER PRIMARY KEY, name)";
            query.exec(queryString);
            
            queryString = "CREATE TABLE unit (id INTEGER PRIMARY KEY, name)";
            query.exec(queryString);
            
            queryString = "CREATE TABLE ingredient (id INTEGER PRIMARY KEY, name)";
            query.exec(queryString);
            
            queryString = "CREATE TABLE recipe_category (id INTEGER PRIMARY KEY, recipe_id, position, category_id)";
            query.exec(queryString);

            queryString = "CREATE TABLE recipe_ingredient (id INTEGER PRIMARY KEY, recipe_id, position, quantity, unit_id, ingredient_id, is_group)";
            query.exec(queryString);

            Config config("zeecookbook");
            config.setGroup("General");
            config.writeEntry("LastCookbook", newDocLnk.file());
        }

        m_pRecipeListView->updateView();
        updateView();
    }
}

void ZeeMainWindow::openLastCookbook()
{
    Config config("zeecookbook");
    config.setGroup("General");
    QString fileName = config.readEntry("LastCookbook");

    if (m_pDatabase->isOpen()) {
        m_pDatabase->close();
    }

    m_pDatabase->setDatabaseName(fileName);
    m_pDatabase->open();

    m_pRecipeListView->updateView();
    updateView();
}

void ZeeMainWindow::openCookbook()
{
    ZeeOpenDialog openDialog(this);
    openDialog.showMaximized();

    if (openDialog.exec() == QDialog::Accepted) {
        DocLnk* docLnk = openDialog.selected();

        if (docLnk) {
            if (m_pDatabase->isOpen()) {
                m_pDatabase->close();
            }

            m_pDatabase->setDatabaseName(docLnk->file());

            if (m_pDatabase->open()) {
                Config config("zeecookbook");
                config.setGroup("General");
                config.writeEntry("LastCookbook", docLnk->file());
            }

            m_pRecipeListView->updateView();
            updateView();
        }
    }
}

void ZeeMainWindow::addRecipe()
{
    if (m_pDatabase->isOpen()) {
        QString queryString = "INSERT INTO recipe VALUES(NULL, '', '', '-1', '')";
        QSqlQuery query(queryString);

        queryString = "SELECT last_insert_rowid()";
        query.exec(queryString);

        if (query.first()) {
            int newRecipeId = query.value(0).toInt();

            ZeeRecipeEditWindow recipeEditWindow(newRecipeId, this);

            recipeEditWindow.showMaximized();
            recipeEditWindow.exec();

            m_pRecipeListView->updateView();
            m_pRecipeListView->select(newRecipeId);

            updateView();
        }
    }
}

void ZeeMainWindow::editRecipe()
{
    if (m_pDatabase->isOpen()) {
        int recipeId = m_pRecipeListView->selected();

        if (recipeId != -1) {
            ZeeRecipeEditWindow recipeEditWindow(recipeId, this);

            recipeEditWindow.showMaximized();
            recipeEditWindow.exec();

            m_pRecipeListView->updateView();
            updateView();
        }
    }
}

void ZeeMainWindow::removeRecipe()
{
    if (m_pDatabase->isOpen()) {
        int recipeId = m_pRecipeListView->selected();

        if (recipeId != -1) {
            QMessageBox messageBox(tr("Remove Recipe"), tr("Do you want to remove\nthe selected recipe?"),
                                   QMessageBox::Information, QMessageBox::Yes,
                                   QMessageBox::No | QMessageBox::Escape | QMessageBox::Default,
                                   QMessageBox::NoButton, this);

            messageBox.setButtonText(QMessageBox::Yes, tr("Yes"));
            messageBox.setButtonText(QMessageBox::No, tr("No"));

            if (messageBox.exec() == QMessageBox::Yes) {
                QString queryString = "DELETE FROM recipe WHERE id = %1";
                QSqlQuery query(queryString.arg(recipeId));

                queryString = "DELETE FROM recipe_category WHERE recipe_id = %1";
                query.exec(queryString.arg(recipeId));

                queryString = "DELETE FROM recipe_ingredient WHERE recipe_id = %1";
                query.exec(queryString.arg(recipeId));

                m_pRecipeListView->updateView();
                updateView();
            }
        }
    }
}

void ZeeMainWindow::viewRecipe()
{
    if (m_pDatabase->isOpen()) {
        int recipeId = m_pRecipeListView->selected();

        if (recipeId != -1) {
            ZeeRecipeViewWindow recipeViewWindow(recipeId, this);

            recipeViewWindow.showMaximized();
            recipeViewWindow.exec();

            m_pRecipeListView->updateView();
            updateView();
        }
    }
}

void ZeeMainWindow::beamRecipe()
{
    QMessageBox messageBox(tr("Beam Recipe"), tr("This feature will be implemented\nin final version!"),
                        QMessageBox::Information, QMessageBox::Ok,
                        QMessageBox::NoButton, QMessageBox::NoButton, this);

    messageBox.setButtonText(QMessageBox::Ok, tr("Ok"));
    messageBox.exec();
}

void ZeeMainWindow::searchTool()
{
    ZeeSearchDialog searchDialog(this);
    searchDialog.showMaximized();

    if (searchDialog.exec() == QDialog::Accepted) {
        QString filter = searchDialog.selected();
        m_pRecipeListView->setFilter(filter);

        m_pRecipeListView->updateView();
        updateView();
    }
}

void ZeeMainWindow::showAllTool()
{
    m_pRecipeListView->setFilter(QString::null);

    m_pRecipeListView->updateView();
    updateView();
}

void ZeeMainWindow::importTool()
{
    ZeeImportDialog importDialog(this);
    importDialog.showMaximized();

    if (importDialog.exec() == QDialog::Accepted) {
        DocLnk* docLnk = importDialog.selected();

        if (docLnk) {
            QFile file(docLnk->file());
            file.open(IO_ReadOnly | IO_Translate);

            QTextStream textStream(&file);

            enum State {
                Unknown,
                InRecipe,
                InRecipeTitle,
                InRecipeCategory,
                InRecipeYield,
                InRecipeIngredient,
                InRecipeDirection
            };

            State state = Unknown;

            QStringList directionList;

            int recipeId = -1;

            QRegExp emptyRegExp = QRegExp("^\\s*$");
            QRegExp headerRegExp1 = QRegExp("^-----.*Meal-Master", false);
            QRegExp headerRegExp2 = QRegExp("^MMMMM.*Meal-Master", false);
            QRegExp titleRegExp = QRegExp("^\\s*Title:\\s", false);
            QRegExp categoryRegExp = QRegExp("^\\s*Categories:\\s", false);
            QRegExp yieldRegExp1 = QRegExp("^\\s*Yield:\\s", false);
            QRegExp yieldRegExp2 = QRegExp("^\\s*Servings:\\s", false);
            QRegExp yieldQuantityRegExp("[0-9]+");
            QRegExp ingredientRegExp = QRegExp("^.......\\s..\\s.+");
            QRegExp ingredientGroupRegExp1 = QRegExp("^------+.+-+$");
            QRegExp ingredientGroupRegExp2 = QRegExp("^MMMMM-+.+-+$");
            QRegExp trailerRegExp1 = QRegExp("^-----\\s*$");
            QRegExp trailerRegExp2 = QRegExp("^MMMMM\\s*$");

            int totalSteps = 0;
            int progress = 0;

            while (!textStream.atEnd()) {
                totalSteps++;
                textStream.readLine();
            }

            file.reset();

            ZeeProgressDialog progressDialog(this);

            progressDialog.setLabelText(tr("Importing recipes..."));
            progressDialog.setProgress(0);
            progressDialog.setTotalSteps(totalSteps);
            progressDialog.show();

            while (!textStream.atEnd()) {
                progress++;
                QString line = textStream.readLine();
                int index, len;

                switch (state) {
                    case Unknown:
                        if ((headerRegExp1.match(line) != -1) || (headerRegExp2.match(line) != -1)) {
                            progressDialog.setProgress(progress);
                            qApp->processEvents();

                            state = InRecipeTitle;
                        }
                        break;
                    case InRecipeTitle:
                        if ((index = titleRegExp.match(line, 0, &len)) != -1) {
                            QString title = zeeFormatString(line.mid(index + len).stripWhiteSpace());

                            QString queryString = "INSERT INTO recipe VALUES(NULL, %1, '', '-1', '')";
                            QSqlQuery query(queryString.arg(title));

                            queryString = "SELECT last_insert_rowid()";
                            query.exec(queryString);

                            if (query.first()) {
                                recipeId = query.value(0).toInt();
                            }

                            state = InRecipeCategory;
                        } else if ((trailerRegExp1.match(line) != -1) || (trailerRegExp2.match(line) != -1)) {
                            state = Unknown;
                        }
                        break;
                    case InRecipeCategory:
                        if ((index = categoryRegExp.match(line, 0, &len)) != -1) {
                            QStringList categoryList = QStringList::split(",", line.mid(index + len));
                            QStringList::Iterator it;

                            for (it = categoryList.begin(); it != categoryList.end(); ++it) {
                                QString categoryName = zeeFormatString((*it).stripWhiteSpace());

                                if (categoryName !=  "''") {
                                    int categoryId = -1;

                                    QString queryString = "SELECT id FROM category WHERE name = %1";
                                    QSqlQuery query(queryString.arg(categoryName));

                                    if (query.first()) {
                                        categoryId = query.value(0).toInt();
                                    } else {
                                        queryString = "INSERT INTO category VALUES(NULL, %1)";
                                        query.exec(queryString.arg(categoryName));

                                        queryString = "SELECT last_insert_rowid()";
                                        query.exec(queryString);

                                        if (query.first()) {
                                            categoryId = query.value(0).toInt();
                                        }
                                    }

                                    queryString = "SELECT COUNT(*) FROM recipe_category WHERE recipe_id = %1";
                                    query.exec(queryString.arg(recipeId));

                                    int count = 0;

                                    if (query.first()) {
                                        count = query.value(0).toInt();
                                    }

                                    queryString = "INSERT INTO recipe_category VALUES(NULL, %1, %2, %3)";
                                    query.exec(queryString.arg(recipeId).arg(count + 1).arg(categoryId));
                                }
                            }

                            state = InRecipeYield;
                        } else if ((trailerRegExp1.match(line) != -1) || (trailerRegExp2.match(line) != -1)) {
                            state = Unknown;
                        }
                        break;
                    case InRecipeYield:
                        if (((index = yieldRegExp1.match(line, 0, &len)) != -1) || ((index = yieldRegExp2.match(line, 0, &len)) != -1)) {
                            QString yield = line.mid(index + len);

                            QString yieldQuantity;
                            QString yieldUnit;
                            int yieldId = -1;

                            if ((index = yieldQuantityRegExp.match(yield, 0, &len)) != -1) {
                                yieldQuantity = zeeFormatString(yield.left(index + len).stripWhiteSpace());
                                yieldUnit = zeeFormatString(yield.mid(index + len).stripWhiteSpace());
                            } else {
                                yieldQuantity = "''";
                                yieldUnit = zeeFormatString(yield.stripWhiteSpace());;
                            }

                            if (yieldUnit !=  "''") {
                                QString queryString = "SELECT id FROM yield WHERE name = %1";
                                QSqlQuery query(queryString.arg(yieldUnit));

                                if (query.first()) {
                                    yieldId = query.value(0).toInt();
                                } else {
                                    queryString = "INSERT INTO yield VALUES(NULL, %1)";
                                    query.exec(queryString.arg(yieldUnit));

                                    queryString = "SELECT last_insert_rowid()";
                                    query.exec(queryString);

                                    if (query.first()) {
                                        yieldId = query.value(0).toInt();
                                    }
                                }
                            }

                            QString queryString = "UPDATE recipe SET yield_quantity = %1, yield_id = %2 WHERE id = %3";
                            QSqlQuery query(queryString.arg(yieldQuantity).arg(yieldId).arg(recipeId));

                            state = InRecipeIngredient;
                        } else if ((trailerRegExp1.match(line) != -1) || (trailerRegExp2.match(line) != -1)) {
                            state = Unknown;
                        }
                        break;
                    case InRecipeIngredient:
                        if ((line.length() <= 39) && (ingredientRegExp.match(line) != -1)) {
                            QString quantity = zeeFormatString(line.left(7).stripWhiteSpace());
                            QString unitName = zeeFormatString(line.mid(8, 2).stripWhiteSpace());
                            QString ingredientName = line.mid(11, 28).stripWhiteSpace();

                            if ((ingredientName !=  "")) {
                                QString queryString = "SELECT recipe_ingredient.id, ingredient_id, name";
                                queryString += " FROM recipe_ingredient, ingredient";
                                queryString += " WHERE recipe_id = %1 AND ingredient_id = ingredient.id AND is_group = 0";
                                queryString += " AND position = (SELECT MAX(position) FROM recipe_ingredient WHERE recipe_id = %2)";
                                QSqlQuery query(queryString.arg(recipeId).arg(recipeId));

                                if (!query.first() || (quantity !=  "''") || (unitName !=  "''") || !ingredientName.startsWith("-")) {
                                    ingredientName = zeeFormatString(ingredientName);
                                    int unitId = -1;
                                    int ingredientId = -1;

                                    if (unitName !=  "''") {
                                        QString queryString = "SELECT id FROM unit WHERE name = %1";
                                        QSqlQuery query(queryString.arg(unitName));

                                        if (query.first()) {
                                            unitId = query.value(0).toInt();
                                        } else {
                                            queryString = "INSERT INTO unit VALUES(NULL, %1)";
                                            query.exec(queryString.arg(unitName));

                                            queryString = "SELECT last_insert_rowid()";
                                            query.exec(queryString);

                                            if (query.first()) {
                                                unitId = query.value(0).toInt();
                                            }
                                        }
                                    }

                                    QString queryString = "SELECT id FROM ingredient WHERE name = %1";
                                    QSqlQuery query(queryString.arg(ingredientName));

                                    if (query.first()) {
                                        ingredientId = query.value(0).toInt();
                                    } else {
                                        queryString = "INSERT INTO ingredient VALUES(NULL, %1)";
                                        query.exec(queryString.arg(ingredientName));

                                        queryString = "SELECT last_insert_rowid()";
                                        query.exec(queryString);

                                        if (query.first()) {
                                            ingredientId = query.value(0).toInt();
                                        }
                                    }

                                    queryString = "SELECT COUNT(*) FROM recipe_ingredient WHERE recipe_id = %1";
                                    query.exec(queryString.arg(recipeId));

                                    int count = 0;

                                    if (query.first()) {
                                        count = query.value(0).toInt();
                                    }

                                    queryString = "INSERT INTO recipe_ingredient VALUES(NULL, %1, %2, %3, %4, %5, 0)";
                                    query.exec(queryString.arg(recipeId).arg(count + 1).arg(quantity).arg(unitId).arg(ingredientId));
                                } else {
                                    int recipeIngredientId = query.value(0).toInt();
                                    int ingredientId = query.value(1).toInt();
                                    ingredientName = zeeFormatString(query.value(2).toString() + ingredientName.replace(0, 1, " "));

                                    queryString = "SELECT COUNT(*) FROM ingredient WHERE id = %1";
                                    query.exec(queryString.arg(ingredientId));

                                    int count = 0;

                                    if (query.first()) {
                                        count = query.value(0).toInt();
                                    }

                                    if (count > 1) {
                                        queryString = "SELECT id FROM ingredient WHERE name = %1";
                                        query.exec(queryString.arg(ingredientName));

                                        if (query.first()) {
                                            ingredientId = query.value(0).toInt();
                                        } else {
                                            queryString = "INSERT INTO ingredient VALUES(NULL, %1)";
                                            query.exec(queryString.arg(ingredientName));

                                            queryString = "SELECT last_insert_rowid()";
                                            query.exec(queryString);

                                            if (query.first()) {
                                                ingredientId = query.value(0).toInt();
                                            }
                                        }

                                        queryString = "UPDATE recipe_ingredient SET ingredient_id = %1 WHERE id = %2";
                                        query.exec(queryString.arg(ingredientId).arg(recipeIngredientId));
                                    } else {
                                        queryString = "UPDATE ingredient SET name = %1 WHERE id = %2";
                                        query.exec(queryString.arg(ingredientName).arg(ingredientId));
                                    }
                                }
                            }
                        } else if ((ingredientGroupRegExp1.match(line) != -1) || (ingredientGroupRegExp2.match(line) != -1)) {
                            QString ingredientName = line.replace(QRegExp("^M+"), "");
                            ingredientName = ingredientName.replace(QRegExp("^-+"), "");
                            ingredientName = ingredientName.replace(QRegExp("-+$"), "");
                            ingredientName = zeeFormatString(ingredientName);

                            if (ingredientName !=  "''") {
                                int ingredientId = -1;

                                QString queryString = "SELECT id FROM ingredient WHERE name = %1";
                                QSqlQuery query(queryString.arg(ingredientName));

                                if (query.first()) {
                                    ingredientId = query.value(0).toInt();
                                } else {
                                    queryString = "INSERT INTO ingredient VALUES(NULL, %1)";
                                    query.exec(queryString.arg(ingredientName));

                                    queryString = "SELECT last_insert_rowid()";
                                    query.exec(queryString);

                                    if (query.first()) {
                                        ingredientId = query.value(0).toInt();
                                    }
                                }

                                queryString = "SELECT COUNT(*) FROM recipe_ingredient WHERE recipe_id = %1";
                                query.exec(queryString.arg(recipeId));

                                int count = 0;

                                if (query.first()) {
                                    count = query.value(0).toInt();
                                }

                                queryString = "INSERT INTO recipe_ingredient VALUES(NULL, %1, %2, '', -1, %3, 1)";
                                query.exec(queryString.arg(recipeId).arg(count + 1).arg(ingredientId));
                            }
                        } else if ((trailerRegExp1.match(line) != -1) || (trailerRegExp2.match(line) != -1)) {
                            state = Unknown;
                        } else if (emptyRegExp.match(line) == -1) {
                            directionList.append(line.stripWhiteSpace());

                            state = InRecipeDirection;
                        }
                        break;
                    case InRecipeDirection:
                        if ((trailerRegExp1.match(line) != -1) || (trailerRegExp2.match(line) != -1)) {
                            QString directions;
                            QStringList::Iterator it;

                            for (it = directionList.begin(); it != directionList.end(); ++it) {
                                directions += (*it);
                            }

                            directions = zeeFormatString(directions);

                            QString queryString = "UPDATE recipe SET direction = %1 WHERE id = %2";
                            QSqlQuery query(queryString.arg(directions).arg(recipeId));

                            directionList.clear();
                            state = Unknown;
                        } else {
                            QString direction = line.stripWhiteSpace();

                            if ((direction.isEmpty()) && (directionList.last() != "\n")) {
                                directionList.append("\n");
                            } else {
                                if (directionList.last() == "\n") {
                                    directionList.append("\n" + direction);
                                } else {
                                    directionList.append(" " + direction);
                                }
                            }
                        }
                        break;
                    default:
                        if ((trailerRegExp1.match(line) != -1) || (trailerRegExp2.match(line) != -1)) {
                            state = Unknown;
                        }
                        break;
                }
            }

            file.close();

            m_pRecipeListView->updateView();
            updateView();
        }
    }
}

void ZeeMainWindow::exportTool()
{
    bool isRecipeSelected = (m_pRecipeListView->selected() != -1);
    int count = m_pRecipeListView->childCount();
    bool hasRecipe = (count > 0);

    ZeeExportDialog exportDialog(hasRecipe, isRecipeSelected, this);
    exportDialog.showMaximized();

    if (exportDialog.exec() == QDialog::Accepted) {
        QString name = exportDialog.selectedName();
        QString format = exportDialog.selectedFormat();

        DocLnk docLnk;
        docLnk.setName(name);

        if (format == tr("Meal-Master")) {
            docLnk.setType("text/plain");
        } else if (format == tr("HTML")) {
            docLnk.setType("text/html");
        }

        FileManager fileManager;
        QIODevice* ioDevice = fileManager.saveFile(docLnk);
        QTextStream textStream(ioDevice);

        switch (exportDialog.selected()) {
            case ZeeExportDialog::Current:
                {
                    int recipeId = m_pRecipeListView->selected();

                    if (recipeId != -1) {
                        if (format == tr("Meal-Master")) {
                            exportMealMaster(recipeId, textStream);
                        } else if (format == tr("HTML")) {
                            exportHtml(recipeId, textStream);
                        }
                    }
                }
                break;
            case ZeeExportDialog::Display:
                {
                    QString filter = m_pRecipeListView->filter();
                    
                    if (format == tr("HTML")) {
                        textStream << "<center><h2>Zee Cookbook</h2></center><ul>\n";

                        QString queryString = "SELECT id, name FROM recipe WHERE %1 ORDER BY name";
                        QSqlQuery query(queryString.arg(filter));

                        while (query.next()) {
                            int recipeId = query.value(0).toInt();
                            QString name = query.value(1).toString();

                            textStream << "<li><a href=\"#" << recipeId << "\">" << name << "</a></li>\n";
                        }

                        textStream << "</ul>\n";
                    }

                    QString queryString = "SELECT id FROM recipe WHERE %1 ORDER BY name";
                    QSqlQuery query(queryString.arg(filter));

                    while (query.next()) {
                        int recipeId = query.value(0).toInt();

                        if (format == tr("Meal-Master")) {
                            exportMealMaster(recipeId, textStream);
                        } else if (format == tr("HTML")) {
                            exportHtml(recipeId, textStream);
                        }
                    }
                }
                break;
            default:
                {
                    if (format == tr("HTML")) {
                        textStream << "<center><h2>Zee Cookbook</h2></center><ul>\n";

                        QString queryString = "SELECT id, name FROM recipe ORDER BY name";
                        QSqlQuery query(queryString);

                        while (query.next()) {
                            int recipeId = query.value(0).toInt();
                            QString name = query.value(1).toString();

                            textStream << "<li><a href=\"#" << recipeId << "\">" << name << "</a></li>\n";
                        }

                        textStream << "</ul>\n";
                    }

                    QString queryString = "SELECT id FROM recipe ORDER BY name";
                    QSqlQuery query(queryString);

                    while (query.next()) {
                        int recipeId = query.value(0).toInt();

                        if (format == tr("Meal-Master")) {
                            exportMealMaster(recipeId, textStream);
                        } else if (format == tr("HTML")) {
                            exportHtml(recipeId, textStream);
                        }
                    }
                }
                break;
        }

        delete ioDevice;
    }
}

void ZeeMainWindow::exportHtml(int recipeId, QTextStream& textStream)
{
    QString queryString = "SELECT name, yield_quantity, yield_id, direction";
    queryString += " FROM recipe WHERE id = %1";
    QSqlQuery query(queryString.arg(recipeId));

    QString name = "";
    QString yieldQuantity = "";
    int yieldId = -1;
    QString direction = "";

    if (query.first()) {
        name = query.value(0).toString();
        yieldQuantity = query.value(1).toString();
        yieldId = query.value(2).toInt();
        direction = query.value(3).toString();
    }

    QString yieldUnit = "";

    if (yieldId != -1) {
        queryString = "SELECT name FROM yield WHERE id = %1";
        query.exec(queryString.arg(yieldId));

        if (query.first()) {
            yieldUnit = query.value(0).toString();
        }
    }

    queryString = "SELECT name FROM recipe_category, category";
    queryString += " WHERE recipe_id = %1 AND category_id = category.id ORDER BY position";
    query.exec(queryString.arg(recipeId));

    QStringList categoryList;

    while (query.next()) {
        QString name = query.value(0).toString();
        categoryList.append(name);
    }

    textStream << "<a name=\"" << recipeId << "\"></a><center><h2>" << name << "</h2></center>\n";
    textStream << "<h3>" << tr("Categories:") << "</h3> " << categoryList.join(", ") << "<br>\n";
    textStream << "<h3>" << tr("Yield:") << "</h3> " << yieldQuantity << " " << yieldUnit << "<br>\n";
    textStream << "<h3>" << tr("Ingredients:") << "</h3><ul>\n";

    queryString = "SELECT quantity, unit.name, ingredient.name, is_group";
    queryString += " FROM recipe_ingredient LEFT JOIN unit ON unit_id = unit.id, ingredient";
    queryString += " WHERE recipe_id = %1 AND ingredient_id = ingredient.id ORDER BY position";
    query.exec(queryString.arg(recipeId));

    while (query.next()) {
        QString quantity = query.value(0).toString();
        QString unit = query.value(1).toString();
        QString name = query.value(2).toString();
        bool isGroup = query.value(3).toInt();

        if (isGroup) {
            textStream << "<b>" << quantity << " " << unit << " " << name << "</b>\n";
        } else {
            textStream << "<li>" << quantity << " " << unit << " " << name << "</li>\n";
        }
    }

    textStream << "</ul><h3>" << tr("Directions:") << "</h3>" << direction.replace(QRegExp("\n"), "<br>\n");
}

void ZeeMainWindow::exportMealMaster(int recipeId, QTextStream& textStream)
{
    QString queryString = "SELECT name, yield_quantity, yield_id, direction";
    queryString += " FROM recipe WHERE id = %1";
    QSqlQuery query(queryString.arg(recipeId));

    QString name = "";
    QString yieldQuantity = "";
    int yieldId = -1;
    QString direction = "";

    if (query.first()) {
        name = query.value(0).toString();
        yieldQuantity = query.value(1).toString();
        yieldId = query.value(2).toInt();
        direction = query.value(3).toString();
    }

    QString yieldUnit = "";

    if (yieldId != -1) {
        queryString = "SELECT name FROM yield WHERE id = %1";
        query.exec(queryString.arg(yieldId));

        if (query.first()) {
            yieldUnit = query.value(0).toString();
        }
    }

    queryString = "SELECT name FROM recipe_category, category";
    queryString += " WHERE recipe_id = %1 AND category_id = category.id ORDER BY position";
    query.exec(queryString.arg(recipeId));

    QStringList categoryList;

    while (query.next()) {
        QString name = query.value(0).toString();
        categoryList.append(name);
    }

    textStream << "-----------------Meal-Master------------\n\n";
    textStream << "      Title: " << name << "\n";
    textStream << " Categories: " << categoryList.join(", ") << "\n";
    textStream << "      Yield: " << yieldQuantity << " " << yieldUnit << "\n\n";

    queryString = "SELECT position, quantity, unit.name, ingredient.name, is_group";
    queryString += " FROM recipe_ingredient LEFT JOIN unit ON unit_id = unit.id, ingredient";
    queryString += " WHERE recipe_id = %1 AND ingredient_id = ingredient.id ORDER BY position";
    query.exec(queryString.arg(recipeId));

    while (query.next()) {
        int position = query.value(0).toInt();
        QString quantity = query.value(1).toString();
        QString unit = query.value(2).toString();
        QString name = query.value(3).toString();
        bool isGroup = query.value(4).toInt();

        if (isGroup) {
            if (position > 1) {
                textStream << "\n";
            }

            QString line = "----------------------------------------";
            int index = (40 - 5 - name.length()) / 2 + 5;
            textStream << line.replace(index, name.length(), name) << "\n";
        } else {
            quantity = quantity.left(7);
            quantity = QString("       ").replace(7 - quantity.length(), quantity.length(), quantity);

            unit = unit.left(2);
            unit = QString("  ").replace(2 - unit.length(), unit.length(), unit);
            
            textStream << quantity << " " << unit << " ";

            QString wrappedName = name.left(28);
            textStream << wrappedName << "\n";
            name = name.remove(0, wrappedName.length());

            while (name.length() > 0) {
                QString wrappedName = name.left(27);
                textStream << "           -" + wrappedName + "\n";
                name = name.remove(0, wrappedName.length());
            }
        }
    }

    textStream << "\n";

    QStringList directionList = QStringList::split("\n", direction, true);
    QStringList::Iterator it;

    QRegExp spaceRegExp = QRegExp("\\s");

    for (it = directionList.begin(); it != directionList.end(); ++it) {
        QString line = (*it);

        if (line.length() > 0) {
            while (line.length() > 0) {
                QString wrappedLine = line.left(75 + 1);

                if (wrappedLine.length() < line.length()) {
                    int index = wrappedLine.findRev(spaceRegExp);
                    wrappedLine = wrappedLine.left(index + 1);
                }

                textStream << "  " + wrappedLine.simplifyWhiteSpace() + "\n";
                line = line.remove(0, wrappedLine.length());
            }
        } else {
            textStream << "\n";
        }
    }

    textStream << "-----\n\n";
}

void ZeeMainWindow::selectRecipe()
{
    updateView();
}
