#include "zeerecipeviewwindow.h"

#include "sql/qsqlquery.h"

#include <qlayout.h>
#include <qtextview.h>

ZeeRecipeViewWindow::ZeeRecipeViewWindow(int recipeId, QWidget* parent, const char* name)
    : ZeeModalWindow(parent, name)
{
    m_recipeId = recipeId;

    setCaption(tr("Zee Cookbook - View Recipe"));

    m_pTextView = new QTextView(this);

    QVBoxLayout* mainLayout = new QVBoxLayout(this, 4, 4);
    mainLayout->addWidget(m_pTextView);

    updateView();
}

ZeeRecipeViewWindow::~ZeeRecipeViewWindow()
{
}

void ZeeRecipeViewWindow::updateView()
{
    QString text;

    QString queryString = "SELECT name, yield_quantity, yield_id, direction";
    queryString += " FROM recipe WHERE id = %1";
    QSqlQuery query(queryString.arg(m_recipeId));

    QString name = "";
    QString yieldQuantity = "";
    int yieldId = -1;
    QString direction = "";

    if (query.first()) {
        name = query.value(0).toString();
        yieldQuantity = query.value(1).toString();
        yieldId = query.value(2).toInt();
        direction = query.value(3).toString();
    }

    QString yieldUnit = "";

    if (yieldId != -1) {
        queryString = "SELECT name FROM yield WHERE id = %1";
        query.exec(queryString.arg(yieldId));

        if (query.first()) {
            yieldUnit = query.value(0).toString();
        }
    }

    queryString = "SELECT name FROM recipe_category, category";
    queryString += " WHERE recipe_id = %1 AND category_id = category.id ORDER BY position";
    query.exec(queryString.arg(m_recipeId));

    QStringList categoryList;

    while (query.next()) {
        QString name = query.value(0).toString();
        categoryList.append(name);
    }

    text += "<center><h2>" + name + "</h2></center>";
    text += "<h3>" + tr("Categories:") + "</h3> " + categoryList.join(", ") + "<br>";
    text += "<h3>" + tr("Yield:") + "</h3> " + yieldQuantity + " " + yieldUnit + "<br>";
    text += "<h3>" + tr("Ingredients:") + "</h3><ul>";

    queryString = "SELECT quantity, unit.name, ingredient.name, is_group";
    queryString += " FROM recipe_ingredient LEFT JOIN unit ON unit_id = unit.id, ingredient";
    queryString += " WHERE recipe_id = %1 AND ingredient_id = ingredient.id ORDER BY position";
    query.exec(queryString.arg(m_recipeId));

    while (query.next()) {
        QString quantity = query.value(0).toString();
        QString unit = query.value(1).toString();
        QString name = query.value(2).toString();
        bool isGroup = query.value(3).toInt();

        if (isGroup) {
            text += "<b>" + quantity + " " + unit + " " + name + "</b>";
        } else {
            text += "<li>" + quantity + " " + unit + " " + name + "</li>";
        }
    }

    text += "</ul><h3>" + tr("Directions:") + "</h3>" + direction.replace(QRegExp("\n"), "<br>");

    m_pTextView->setText(text);
}
